#pragma once
#include <clippy/config.hpp>

#include <stdexcept>
#include <utils/lock_file.hpp>

#include <vector>
#include <filesystem>
#include <optional>

#include <jsoncons/json.hpp>

struct Project {
  Project() {}

  Project(std::filesystem::path root_project, std::filesystem::path configuration_file)
      : root_project(root_project), configuration_file(configuration_file) {}

  std::strong_ordering operator<=>(const Project&) const = default;

  Config GetConfig() {
    return Config{configuration_file, root_project};
  }

  std::filesystem::path root_project;
  std::filesystem::path configuration_file;
  std::optional<std::string> name;
};

class ProjectList {
 public:
  ProjectList(std::filesystem::path path) : path_(std::move(path)), lock_file_(path_) {
    Load();
  }

  Config GetNewConfig(const std::filesystem::path& config_directory);

  const std::vector<Project>& GetProjects() const {
    return projects_;
  }

  std::optional<Project> GetCurrentProject() {
    auto* p = GetCurrentProject_();
    if (p) {
      return *p;
    } else {
      return std::nullopt;
    }
  }

  void SetNameCurrentProject(const std::string& name) {
    auto* p = GetCurrentProject_();
    if (p) {
      p->name = name;
      SaveConfig();
    } else {
      throw std::logic_error("Not exists current project");
    }
  }

 private:
  Project* GetCurrentProject_();

  void OldLoadConfig(const std::string&);
  void SaveConfig();

  void Load();
  void LoadConfig(const jsoncons::json&);
  void LoadWithoutLock();

 private:
  std::filesystem::path path_;
  utils::filesystem::LockFile lock_file_;

  std::vector<Project> projects_;
};
